@@ -1,29 +1,30 @@
 <?php
 /**
- * $Header: /repository/pear/Log/Log.php,v 1.63 2006/05/23 05:59:58 jon Exp $
+ * $Header$
  * $Horde: horde/lib/Log.php,v 1.15 2000/06/29 23:39:45 jon Exp $
  *
- * @version $Revision: 1.63 $
+ * @version $Revision: 293929 $
  * @package Log
  */
 
-define('PEAR_LOG_EMERG',    0);     /** System is unusable */
-define('PEAR_LOG_ALERT',    1);     /** Immediate action required */
-define('PEAR_LOG_CRIT',     2);     /** Critical conditions */
-define('PEAR_LOG_ERR',      3);     /** Error conditions */
-define('PEAR_LOG_WARNING',  4);     /** Warning conditions */
-define('PEAR_LOG_NOTICE',   5);     /** Normal but significant */
-define('PEAR_LOG_INFO',     6);     /** Informational */
-define('PEAR_LOG_DEBUG',    7);     /** Debug-level messages */
+define('PEAR_LOG_EMERG',    0);     /* System is unusable */
+define('PEAR_LOG_ALERT',    1);     /* Immediate action required */
+define('PEAR_LOG_CRIT',     2);     /* Critical conditions */
+define('PEAR_LOG_ERR',      3);     /* Error conditions */
+define('PEAR_LOG_WARNING',  4);     /* Warning conditions */
+define('PEAR_LOG_NOTICE',   5);     /* Normal but significant */
+define('PEAR_LOG_INFO',     6);     /* Informational */
+define('PEAR_LOG_DEBUG',    7);     /* Debug-level messages */
 
-define('PEAR_LOG_ALL',      bindec('11111111'));  /** All messages */
-define('PEAR_LOG_NONE',     bindec('00000000'));  /** No message */
+define('PEAR_LOG_ALL',      0xffffffff);    /* All messages */
+define('PEAR_LOG_NONE',     0x00000000);    /* No message */
 
 /* Log types for PHP's native error_log() function. */
-define('PEAR_LOG_TYPE_SYSTEM',  0); /** Use PHP's system logger */
-define('PEAR_LOG_TYPE_MAIL',    1); /** Use PHP's mail() function */
-define('PEAR_LOG_TYPE_DEBUG',   2); /** Use PHP's debugging connection */
-define('PEAR_LOG_TYPE_FILE',    3); /** Append to a file */
+define('PEAR_LOG_TYPE_SYSTEM',  0); /* Use PHP's system logger */
+define('PEAR_LOG_TYPE_MAIL',    1); /* Use PHP's mail() function */
+define('PEAR_LOG_TYPE_DEBUG',   2); /* Use PHP's debugging connection */
+define('PEAR_LOG_TYPE_FILE',    3); /* Append to a file */
+define('PEAR_LOG_TYPE_SAPI',    4); /* Use the SAPI logging handler */
 
 /**
  * The Log:: class implements both an abstraction for various logging
@@ -40,7 +41,7 @@ class Log
      * Indicates whether or not the log can been opened / connected.
      *
      * @var boolean
-     * @access private
+     * @access protected
      */
     var $_opened = false;
 
@@ -48,7 +49,7 @@ class Log
      * Instance-specific unique identification number.
      *
      * @var integer
-     * @access private
+     * @access protected
      */
     var $_id = 0;
 
@@ -56,7 +57,7 @@ class Log
      * The label that uniquely identifies this set of log messages.
      *
      * @var string
-     * @access private
+     * @access protected
      */
     var $_ident = '';
 
@@ -64,7 +65,7 @@ class Log
      * The default priority to use when logging an event.
      *
      * @var integer
-     * @access private
+     * @access protected
      */
     var $_priority = PEAR_LOG_INFO;
 
@@ -72,7 +73,7 @@ class Log
      * The bitmask of allowed log levels.
      *
      * @var integer
-     * @access private
+     * @access protected
      */
     var $_mask = PEAR_LOG_ALL;
 
@@ -80,7 +81,7 @@ class Log
      * Holds all Log_observer objects that wish to be notified of new messages.
      *
      * @var array
-     * @access private
+     * @access protected
      */
     var $_listeners = array();
 
@@ -89,7 +90,7 @@ class Log
      * "line format" strings.
      *
      * @var array
-     * @access private
+     * @access protected
      */
     var $_formatMap = array('%{timestamp}'  => '%1$s',
                             '%{ident}'      => '%2$s',
@@ -98,9 +99,9 @@ class Log
                             '%{file}'       => '%5$s',
                             '%{line}'       => '%6$s',
                             '%{function}'   => '%7$s',
+                            '%{class}'      => '%8$s',
                             '%\{'           => '%%{');
 
-
     /**
      * Attempts to return a concrete Log instance of type $handler.
      *
@@ -138,13 +139,13 @@ class Log
          * a failure as fatal.  The caller may have already included their own
          * version of the named class.
          */
-        if (!class_exists($class)) {
+        if (!class_exists($class, false)) {
             include_once $classfile;
         }
 
         /* If the class exists, return a new instance of it. */
-        if (class_exists($class)) {
-            $obj = &new $class($name, $ident, $conf, $level);
+        if (class_exists($class, false)) {
+            $obj = new $class($name, $ident, $conf, $level);
             return $obj;
         }
 
@@ -390,7 +391,7 @@ class Log
      *
      * @return string           The string representation of the message.
      *
-     * @access private
+     * @access protected
      */
     function _extractMessage($message)
     {
@@ -407,20 +408,22 @@ class Log
             } else if (method_exists($message, 'tostring')) {
                 $message = $message->toString();
             } else if (method_exists($message, '__tostring')) {
-                if (version_compare(PHP_VERSION, '5.0.0', 'ge')) {
-                    $message = (string)$message;
-                } else {
-                    $message = $message->__toString();
-                }
+                $message = (string)$message;
             } else {
-                $message = print_r($message, true);
+                $message = var_export($message, true);
             }
         } else if (is_array($message)) {
             if (isset($message['message'])) {
-                $message = $message['message'];
+                if (is_scalar($message['message'])) {
+                    $message = $message['message'];
+                } else {
+                    $message = var_export($message['message'], true);
+                }
             } else {
-                $message = print_r($message, true);
+                $message = var_export($message, true);
             }
+        } else if (is_bool($message) || $message === NULL) {
+            $message = var_export($message, true);
         }
 
         /* Otherwise, we assume the message is a string. */
@@ -434,8 +437,9 @@ class Log
      * @param   int     $depth  The initial number of frames we should step
      *                          back into the trace.
      *
-     * @return  array   Array containing three strings: the filename, the line,
-     *                  and the function name from which log() was called.
+     * @return  array   Array containing four strings: the filename, the line,
+     *                  the function name, and the class name from which log()
+     *                  was called.
      *
      * @access  private
      * @since   Log 1.9.4
@@ -443,14 +447,16 @@ class Log
     function _getBacktraceVars($depth)
     {
         /* Start by generating a backtrace from the current call (here). */
-        $backtrace = debug_backtrace();
+        $bt = debug_backtrace();
 
         /*
          * If we were ultimately invoked by the composite handler, we need to
          * increase our depth one additional level to compensate.
          */
-        if (strcasecmp(@$backtrace[$depth+1]['class'], 'Log_composite') == 0) {
+        $class = isset($bt[$depth+1]['class']) ? $bt[$depth+1]['class'] : null;
+        if ($class !== null && strcasecmp($class, 'Log_composite') == 0) {
             $depth++;
+            $class = isset($bt[$depth + 1]) ? $bt[$depth + 1]['class'] : null;
         }
 
         /*
@@ -460,9 +466,9 @@ class Log
          * further back to find the name of the encapsulating function from
          * which log() was called.
          */
-        $file = @$backtrace[$depth]['file'];
-        $line = @$backtrace[$depth]['line'];
-        $func = @$backtrace[$depth + 1]['function'];
+        $file = isset($bt[$depth])     ? $bt[$depth]['file'] : null;
+        $line = isset($bt[$depth])     ? $bt[$depth]['line'] : 0;
+        $func = isset($bt[$depth + 1]) ? $bt[$depth + 1]['function'] : null;
 
         /*
          * However, if log() was called from one of our "shortcut" functions,
@@ -470,9 +476,10 @@ class Log
          */
         if (in_array($func, array('emerg', 'alert', 'crit', 'err', 'warning',
                                   'notice', 'info', 'debug'))) {
-            $file = @$backtrace[$depth + 1]['file'];
-            $line = @$backtrace[$depth + 1]['line'];
-            $func = @$backtrace[$depth + 2]['function'];
+            $file = isset($bt[$depth + 1]) ? $bt[$depth + 1]['file'] : null;
+            $line = isset($bt[$depth + 1]) ? $bt[$depth + 1]['line'] : 0;
+            $func = isset($bt[$depth + 2]) ? $bt[$depth + 2]['function'] : null;
+            $class = isset($bt[$depth + 2]) ? $bt[$depth + 2]['class'] : null;
         }
 
         /*
@@ -483,8 +490,8 @@ class Log
             $func = '(none)';
         }
 
-        /* Return a 3-tuple containing (file, line, function). */
-        return array($file, $line, $func);
+        /* Return a 4-tuple containing (file, line, function, class). */
+        return array($file, $line, $func, $class);
     }
 
     /**
@@ -493,17 +500,17 @@ class Log
      *
      * @return  string  Formatted log string.
      *
-     * @access  private
+     * @access  protected
      * @since   Log 1.9.4
      */
     function _format($format, $timestamp, $priority, $message)
     {
         /*
          * If the format string references any of the backtrace-driven
-         * variables (%5, %6, %7), generate the backtrace and fetch them.
+         * variables (%5 %6,%7,%8), generate the backtrace and fetch them.
          */
-        if (preg_match('/%[567]/', $format)) {
-            list($file, $line, $func) = $this->_getBacktraceVars(2);
+        if (preg_match('/%[5678]/', $format)) {
+            list($file, $line, $func, $class) = $this->_getBacktraceVars(2);
         }
 
         /*
@@ -518,7 +525,8 @@ class Log
                        $message,
                        isset($file) ? $file : '',
                        isset($line) ? $line : '',
-                       isset($func) ? $func : '');
+                       isset($func) ? $func : '',
+                       isset($class) ? $class : '');
     }
 
     /**
@@ -528,6 +536,7 @@ class Log
      *
      * @return string           The string representation of $level.
      *
+     * @access  public
      * @since   Log 1.0
      */
     function priorityToString($priority)
@@ -556,6 +565,7 @@ class Log
      * @return string           The PEAR_LOG_* integer contstant corresponding
      *                          the the specified priority name.
      *
+     * @access  public
      * @since   Log 1.9.0
      */
     function stringToPriority($name)
@@ -686,7 +696,7 @@ class Log
      * @return boolean  True if the given priority is included in the current
      *                  log mask.
      *
-     * @access  private
+     * @access  protected
      * @since   Log 1.7.0
      */
     function _isMasked($priority)
@@ -772,7 +782,7 @@ class Log
      *
      * @param array     $event      A hash describing the log event.
      *
-     * @access private
+     * @access protected
      */
     function _announce($event)
     {
